@@ -9,6 +9,7 @@ import collections.abc
 import functools
 import importlib
 import importlib.util
+import inspect
 import itertools
 import os
 import re
@@ -271,6 +272,34 @@ def repr(obj, htchar=' ', lfchar='\n', indent=4, basic_offset=0):
     return f'{type(obj).__name__}({r})@{hex(id(obj))}'
 
 
+def attrs(obj):
+    '''Inspect object and return its attributes and their values.
+
+    This function returns also any descriptors found at the owner class,
+    with the exception of descriptors without an assigned value, which are
+    expected to raise an ``AttributeError``.
+
+    :arg obj: The object to inspect.
+    :returns: an iterator over ``(attr_name, value)`` tuples
+
+    :meta private:
+    '''
+
+    ret = dict(obj.__dict__)
+
+    # Look for descriptors
+    for cls in type(obj).mro():
+        for attr in cls.__dict__:
+            if inspect.isdatadescriptor(cls.__dict__[attr]):
+                try:
+                    ret[attr] = getattr(obj, attr)
+                except AttributeError:
+                    # Pass if the descriptor does not have an assigned value
+                    pass
+
+    return ret
+
+
 def _is_builtin_type(cls):
     # NOTE: The set of types is copied from the copy.deepcopy() implementation
     builtin_types = (type(None), int, float, bool, complex, str, tuple,
@@ -536,10 +565,12 @@ def find_modules(substr, environ_mapping=None):
 
     .. code:: python
 
-       @rfm.parameterized_test(*find_modules('netcdf'))
+       @rfm.simple_test
        class MyTest(rfm.RegressionTest):
-           def __init__(self, s, e, m):
-               self.descr = f'{s}, {e}, {m}'
+           module_info = parameter(find_modules('netcdf'))
+
+           def __init__(self):
+               s, e, m = self.module_info
                self.valid_systems = [s]
                self.valid_prog_environs = [e]
                self.modules = [m]
@@ -554,15 +585,17 @@ def find_modules(substr, environ_mapping=None):
     .. code:: python
 
        my_find_modules = functools.partial(find_modules, environ_mapping={
-           r'.*CrayGNU.*': {'PrgEnv-gnu'},
-           r'.*CrayIntel.*': {'PrgEnv-intel'},
-           r'.*CrayCCE.*': {'PrgEnv-cray'}
+           r'.*CrayGNU.*': 'PrgEnv-gnu',
+           r'.*CrayIntel.*': 'PrgEnv-intel',
+           r'.*CrayCCE.*': 'PrgEnv-cray'
        })
 
-       @rfm.parameterized_test(*my_find_modules('GROMACS'))
+       @rfm.simple_test
        class MyTest(rfm.RegressionTest):
-           def __init__(self, s, e, m):
-               self.descr = f'{s}, {e}, {m}'
+           module_info = parameter(my_find_modules('GROMACS'))
+
+           def __init__(self):
+               s, e, m = self.module_info
                self.valid_systems = [s]
                self.valid_prog_environs = [e]
                self.modules = [m]
@@ -643,6 +676,7 @@ class ScopedDict(UserDict):
     ``d['*:k1']`` are all equivalent.
     If you try to retrieve a whole scope, e.g., ``d['a:b']``,
     :class:`KeyError` will be raised.
+    For retrieving scopes, you should use the :func:`scope` function.
 
     Key deletion follows the same resolution mechanism as key retrieval,
     except that you are allowed to delete whole scopes. For example, ``del
@@ -698,6 +732,29 @@ class ScopedDict(UserDict):
             for k, v in scope_dict.items():
                 self.data[scope][k] = v
 
+    def scope(self, name):
+        '''Retrieve a whole scope.
+
+        :arg scope: The name of the scope to retrieve.
+        :returns: A dictionary with the keys that are within the requested
+            scope.
+        '''
+
+        ret = {}
+        curr_scope = name
+        while curr_scope is not None:
+            if curr_scope in self.data:
+                for k, v in self.data[curr_scope].items():
+                    if k not in ret:
+                        ret[k] = v
+
+            if curr_scope == self._global_scope:
+                curr_scope = None
+            else:
+                curr_scope = self._parent_scope(curr_scope)
+
+        return ret
+
     def __str__(self):
         # just return the internal dictionary
         return str(self.data)
